from __future__ import print_function

import onnx
import onnx.helper as oh
from onnx import checker
from get_engine import get_engine
import numpy as np
import tensorrt as trt
import pycuda.driver as cuda
import pycuda.autoinit
import sys
import os
import common

AXIS = 1
#SHAPE = [1, 3, 4, 5]
SHAPE = [1, 1, 3, 4, 5]
ONNX_FILE = "softmax_test.onnx"

# Generate ONNX model
def generate_model(io_shape=SHAPE, axis=AXIS):
    in_tensor = [
        oh.make_tensor_value_info("Input", onnx.TensorProto.FLOAT, io_shape)
    ]
    out_tensor = [
        oh.make_tensor_value_info("Output", onnx.TensorProto.FLOAT, io_shape)
    ]
    nodes = []
    nodes.append(oh.make_node("Softmax", axis=axis, \
        inputs=["Input"], outputs=["Output"]))
    graph = oh.make_graph(nodes, "Test Graph", in_tensor, out_tensor)
    checker.check_graph(graph)
    model = oh.make_model(graph, producer_name="MACNICA", \
        producer_version="0.1")
    checker.check_model(model)
    with open(ONNX_FILE, "wb") as f:
        f.write(model.SerializeToString())

# Test model
def test_model(io_shape=SHAPE):

    # Load the model into TensorRT
    with get_engine(ONNX_FILE) as engine, \
        engine.create_execution_context() as context:

        # Allocate buffer memory for TensorRT
        inputs, outputs, bindings, stream = common.allocate_buffers(engine)
        
        # Prepare input data
        num_elements = np.zeros(io_shape).size
        values = np.arange(num_elements, \
            dtype=np.float32).reshape(tuple(io_shape)) / 100
        print('Input:')
        print(values)
        
        # Inference
        inputs[0].host = values
        trt_outputs = common.do_inference(context, bindings=bindings, \
            inputs=inputs, outputs=outputs, stream=stream)
            
        print('Output:')
        print(trt_outputs[0].reshape(io_shape[1:]))
        
        print(np.sum(trt_outputs[0]))
        
# Main function
def main():
    generate_model(SHAPE, AXIS)
    test_model(SHAPE)
  
if __name__ == "__main__":
    main()
